/*
 * Common java library used for Coffee Quest 4 & Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.common.sprites;

import java.awt.*;
import java.awt.image.*;

/**
 * A Text based sprite with a variety of effects that can be applied to the
 * text.
 * 
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: June 20, 1998
 */
public class TextSprite implements SpriteProducer {
// *** CONSTANTS ***
  // Text Styles
  public static final int NORMAL = 0;
  public static final int SHADOW = 1;
  public static final int OUTLINE = 2;
  public static final int TWOTONE = 3;

  // Text Alignment
  public static final int ALIGNLEFT = 0;
  public static final int ALIGNRIGHT = 1;
  public static final int ALIGNCENTER = 2;

  public static final Font DEFAULTFONT = new Font("TimesRoman", Font.PLAIN, 12);
// *** VARIABLES ***
  Font m_font;
        // the font to draw the text with
  String m_sText;
        // The string to be displayed
  Color m_colorText, m_colorOutline, m_colorTwoTone, m_colorBackdrop;
        // the colors to use when drawing the text
  int m_nStyle, m_nAmount, m_nAlign;
        // type of text to draw and amount of that style
  int m_nWidth, m_nHeight;
        // Size of the text area
  boolean m_bUseBackground;
        // Determins if background is solid color

// *** CONSTRUCTORS ***
  // default constructor.  Uses 12 point font with red text.
  public TextSprite() {
    m_font = DEFAULTFONT;//new Font("TimesRoman", Font.PLAIN, 12);
    m_colorText = Color.red;
    m_colorOutline = Color.black;
    m_colorTwoTone = Color.black;
    m_colorBackdrop = Color.black;
    m_nStyle = NORMAL;
    m_nWidth = m_nHeight = 20;
    m_bUseBackground = false;
    m_nAlign = 0;
    m_sText = " ";
  }

  // Constructor using provided font, string, and text color.
  public TextSprite(Font f, String sText, Color cText, int w, int h) {
    m_font = f;
    m_colorText = cText;
    m_colorOutline = Color.black;
    m_colorTwoTone = Color.black;
    m_colorBackdrop = Color.black;
    m_nStyle = NORMAL;
    m_nWidth = w;
    m_nHeight = h;
    m_bUseBackground = false;
    m_nAlign = 0;
    m_sText = sText;
  }

// *** MODIFICATION FUNCTIONS ***
  // sets the text color
  public void setTextColor(Color cText){
    m_colorText = cText;
  }

   // sets the outline color
   public void setOutlineColor(Color cOutline){
       m_colorOutline = cOutline;
   }

  // sets the two-tone color
  public void setTwoToneColor(Color cTwoTone) {
    m_colorTwoTone = cTwoTone;
  }

  public void setBackground(boolean bState, Color color) {
    m_bUseBackground = bState;
    m_colorBackdrop = color;
  }

  // Sets the style to plain using provided color
  public void setPlain(Color cText) {
    m_colorText = cText;
    m_nStyle = NORMAL;
  }

  public void setShadow(Color cText, Color cShadow, int nGap) {
    m_colorText = cText;
    m_colorOutline = cShadow;
    m_nAmount = nGap;
    m_nStyle = SHADOW;
  }

  public void setOutline(Color cText, Color cOutline, int nGap) {
    m_colorText = cText;
    m_colorOutline = cOutline;
    m_nAmount = nGap;
    m_nStyle = OUTLINE;
  }

  public void setTwoTone(Color cText, Color cTop, Color cBottom, int nGap) {
    m_colorText = cText;
    m_colorOutline = cTop;
    m_colorTwoTone = cBottom;
    m_nAmount = nGap;
    m_nStyle = TWOTONE;
  }

  // sets the font to use when drawing the text
  public void setFont(Font f){
    m_font = f;
  }

  public void setText(String s, int nAlignment) {
    m_sText = s;
    m_nAlign = nAlignment;
  }

  public void setSize(int nW, int nH) {
    m_nWidth = nW;
    m_nHeight = nH;
  }

// *** INFORMATION FUNCTIONS ***
	// Returns the width of the component
	public int getWidth() {
    return m_nWidth;
  }

  // Returns the width of the text taking the style into account
  public int getTextWidth(FontMetrics fm) {
    int result;

    result = fm.stringWidth(m_sText);
    if (m_nStyle != NORMAL) {
      result += m_nAmount;
      if (m_nStyle > SHADOW)
        result += m_nAmount;
    }
    return result;
  }

	// Returns the height of the component
	public int getHeight() {
    return m_nHeight;
  }

	// Returns the height of the text taking the style into account
	public int getTextHeight(FontMetrics fm) {
    int result;

    result = fm.getMaxAscent() + fm.getMaxDescent();
    if (m_nStyle != NORMAL) {
      result += m_nAmount;
      if (m_nStyle > SHADOW)
        result += m_nAmount;
    }
    return result;
  }


// *** DRAWING FUNCTIONS ***
  // Paint the text using the style.  x and y should be proper coordinates
  // taking the Alignment and width/height into account.
  protected void textPaint(Graphics g, int x, int y) {
    int cntrX, cntrY;

    // Prepare for drawing
    Font fontOld = g.getFont();
    Color colorOld = g.getColor();
    g.setFont(m_font);

    switch (m_nStyle) {
      case NORMAL :
        g.setColor(m_colorText);
        g.drawString(m_sText, x, y);
        break;
      case SHADOW :
        g.setColor(m_colorOutline);
        g.drawString(m_sText, x+m_nAmount, y+m_nAmount);
        g.setColor(m_colorText);
        g.drawString(m_sText, x, y);
        break;
      case OUTLINE :
        g.setColor(m_colorOutline);
        for (cntrX = 0; cntrX < (2 * m_nAmount + 1); ++cntrX)
          for (cntrY = 0; cntrY < (2 * m_nAmount + 1); ++cntrY)
            g.drawString(m_sText, x+cntrX, y+cntrY);
        g.setColor(m_colorText);
        g.drawString(m_sText, x + m_nAmount, y + m_nAmount);
        break;
      case TWOTONE :
        g.setColor(m_colorOutline);
        for (cntrX = 0; cntrX < m_nAmount; ++cntrX)
          g.drawString(m_sText, x+cntrX, y+cntrX);
        g.setColor(m_colorTwoTone);
        for (cntrX = m_nAmount+1; cntrX < (m_nAmount*2+1); ++cntrX)
          g.drawString(m_sText, x+cntrX, y+cntrX);
        g.setColor(m_colorText);
        g.drawString(m_sText, x+m_nAmount, y+m_nAmount);
        break;
    }

    // clean up
    g.setFont(fontOld);
    g.setColor(colorOld);
  }

	// Handles drawing contents at provided coordinates
	public void relativePaint(Graphics g, int nX, int nY, ImageObserver observer) {
    FontMetrics fm;
    int y, w, h, gapx, gapy;

    fm = g.getFontMetrics(m_font);
    w = getWidth();
    h = getHeight();
    if (m_bUseBackground) {
      Color oldColor = g.getColor();
      g.setColor(m_colorBackdrop);
      g.fillRect(nX, nY, w, h);
      g.setColor(oldColor);
    }
    gapx = w - getTextWidth(fm);
    gapy = h - getTextHeight(fm);
    // Find vertical center
    y = nY + fm.getMaxAscent() + gapy / 2;

    // Do the drawing based on the mode
    switch (m_nAlign) {
      case ALIGNLEFT :
        textPaint(g, nX + 1, y);
        break;
      case ALIGNRIGHT :
        textPaint(g, nX+gapx-1, y);
        break;
      case ALIGNCENTER :
        textPaint(g, nX+gapx/2, y);
        break;
    }
  }
}
