/*
 * Common java library used for Coffee Quest 4 & Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.common.zones;

import java.awt.*;
import java.awt.image.*;
import com.blazinggames.common.sprites.*;

/**
 * A universal item holder.  Due to deficiencies with Java 1.1, the actual
 * drawing of the items is going to have to be done by the owner of
 * the component.  Drag events will tell them when they have to redraw the item
 * at a different spot.  The getItemX and getItemY functions will always tell
 * the owner where the CENTER of the item should be.
 *  
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: March 4, 1998
 */
public class ItemHolder extends Zone {
// *** CONSTANTS ***

// *** VARIABLES ***
	SpriteProducer m_pItem;
	int nItemType;
			// The Item that this component is holding
	int m_nHoldMask;
			// Determin types of items that can be put here
	int m_nItemX, m_nItemY, m_nOffsetX, m_nOffsetY;
			// position of the item
	int m_nOldItemX, m_nOldItemY, m_nOldItemX2, m_nOldItemY2;
			// Area that pointer moved through since last clearing;
	Color colorHighlight, colorNormal, colorSelected;
			// Colors used for drawing the zone.
	int m_nBorderSize;
			// how thick the border should be.
	boolean m_bItemNeedsToBeDrawn, m_bDragDrawState;

// *** CONSTRUCTOR ***
	public ItemHolder(int nID, int nX, int nY, int nW, int nH, int nHoldMask) {
		super(nID, nX, nY, nW, nH, Zone.MASK_HIGHLIGHT | Zone.MASK_SELECTED |
					Zone.MASK_DRAGGED | Zone.MASK_RELEASED |
					Zone.MASK_DROPABLE | Zone.MASK_RECEIVER );
		m_nHoldMask = nHoldMask;
		m_nItemX = centerX();
		m_nItemY = centerY();

		colorHighlight = Color.orange;
		colorNormal = Color.blue;
		colorSelected = Color.red;

		m_nBorderSize = 3;

		m_pItem = null;
		nItemType = 0;
		m_bItemNeedsToBeDrawn = true;
		m_bDragDrawState = false;
	}

// *** MODIFICATION ***
	// Set the types of things that this holder can hold
	public void setHoldMask(int nHoldMask) {
		m_nHoldMask = nHoldMask;
	}

	// Set the color of the holder
	public void setColors(Color colorNorm, Color colorHigh, Color colorSel) {
		colorHighlight = colorHigh;
		colorNormal = colorNorm;
		colorSelected = colorSel;
	}

	// set the border size of the holder
	public void setBorderSize(int nSize) {
		m_nBorderSize = nSize;
	}

	// sets the item that the holder is holding
	// returns false if the item is a type the holder can't use.
	public boolean setItem(SpriteProducer pItem, int nType){
		if (pItem == null) {
			m_pItem = null;
			nItemType = 0;
			return true;
		}
		else {
			if ( (m_nHoldMask & nType) != 0) {
				m_pItem = pItem;
				nItemType = nType;
				return true;
			}
			else
				return false;
		}
	}

	public void clearDragDrawState() {
		m_bDragDrawState = false;
	}

// *** ACTION ***
	public boolean swapHolder(ItemHolder ihSwap) {
		SpriteProducer pTemp = ihSwap.getItem();
		int nTemp = ihSwap.getItemType();

		if ( ( (pTemp == null) || ((nTemp & m_nHoldMask) != 0) ) &&
				((m_pItem == null) || ((nItemType & ihSwap.m_nHoldMask) != 0))){
			ihSwap.setItem(m_pItem,nItemType);
			setItem(pTemp, nTemp);
			return true;
		}
		return false;
	}

// *** INFORMATION ***
	// returns the center X point of where the item should be drawn
	public int getItemX() {
		return m_nItemX;
	}

	// returns the center Y point of where the item should be drawn
	public int getItemY() {
		return m_nItemY;
	}

	// returns the center X point of where the item should be drawn
	public int getOldItemX() {
		return m_nOldItemX;
	}

	// returns the center Y point of where the item should be drawn
	public int getOldItemY() {
		return m_nOldItemY;
	}

	// returns the center X point of where the item should be drawn
	public int getOldItemX2() {
		return m_nOldItemX2;
	}

	// returns the center Y point of where the item should be drawn
	public int getOldItemY2() {
		return m_nOldItemY2;
	}

	public int getItemType() {
		return nItemType;
	}

	public SpriteProducer getItem() {
		return m_pItem;
	}

	// Returns true if item needs to be drawn
	public boolean needToDrawItem() {
		if (m_pItem == null)
			return false;
		else
			return m_bItemNeedsToBeDrawn;
	}

	public boolean checkDragDrawState() {
		return m_bDragDrawState;
	}

	// returns rectangle with coordinates of the dirty rectangle due to dragging
	public Rectangle getDragDropDirtyRect() {
		int nX, nY, nW, nH, nTemp;

		if (m_pItem == null)
			return new Rectangle();
		nTemp = m_pItem.getWidth();
		nX = m_nOldItemX - nTemp / 2;
		nW = m_nOldItemX2 - m_nOldItemX + nTemp;
		nTemp = m_pItem.getHeight();
		nY = m_nOldItemY - nTemp / 2;
		nH = m_nOldItemY2 - m_nOldItemY + nTemp;
		return new Rectangle(nX, nY, nW, nH);
	}

	// returns the horizontal center of the zone
	protected int centerX() {
		return m_nX + getWidth() / 2;
	}

	// returns the vertical center of the zone
	protected int centerY() {
		return m_nY + getHeight() / 2;
	}

// *** EVENT HANDLERS ***
	public int mouseDown(int nX, int nY) {
		int nEvent = super.mouseDown(nX, nY);

		if (nEvent == ZoneListener.ZN_SELECTED) {
			m_nOffsetX = centerX() - nX;
			m_nOffsetY = centerY() - nY;
			m_nOldItemX = nX + m_nOffsetX;
			m_nOldItemY = nY + m_nOffsetY;
		}
		return nEvent;
	}

	public int mouseUp(int nX, int nY) {
		if (m_bDragDrawState) {
			m_nOldItemX = Math.min(m_nOldItemX, m_nItemX);
			m_nOldItemX2 = Math.max(m_nOldItemX2, m_nItemX);
			m_nOldItemY = Math.min(m_nOldItemY, m_nItemY);
			m_nOldItemY2 = Math.max(m_nOldItemY2, m_nItemY);
		}
		else {
			m_nOldItemX = m_nOldItemX2 = m_nItemX;
			m_nOldItemY = m_nOldItemY2 = m_nItemY;
			m_bDragDrawState = true;
		}
		m_nItemX = centerX();
		m_nItemY = centerY();
		return super.mouseUp(nX, nY);
	}

	// Moves the drag position
	public int mouseDrag(int nX, int nY) {
		if ( (m_nState == STATE_SELECTED) || (m_nState == STATE_DRAGGED) ) {
			if (m_bDragDrawState) {
				m_nOldItemX = Math.min(m_nOldItemX, m_nItemX);
				m_nOldItemX2 = Math.max(m_nOldItemX2, m_nItemX);
				m_nOldItemY = Math.min(m_nOldItemY, m_nItemY);
				m_nOldItemY2 = Math.max(m_nOldItemY2, m_nItemY);
			}
			else {
				m_nOldItemX = m_nOldItemX2 = m_nItemX;
				m_nOldItemY = m_nOldItemY2 = m_nItemY;
				m_bDragDrawState = true;
			}
			m_nItemX = nX + m_nOffsetX;
			m_nItemY = nY + m_nOffsetY;
		}
		return super.mouseDrag(nX, nY);
	}

// *** DRAWING ***
	// Fully draws the component
	public int paint(Graphics g, ImageObserver observer) {
    if (!isVisible())
      return 0;

		boolean b3DMode = true;
    Color oldColor = g.getColor();

		super.paint(g, observer);
		switch (m_nState) {
			case STATE_DISABLED:
				g.setColor(Color.gray);
				break;
			case STATE_ACTIVE:
				g.setColor(colorNormal);
				break;
			case STATE_HIGHLIGHTED:
				g.setColor(colorHighlight);
				break;
			case STATE_SELECTED:
			case STATE_DRAGGED:
			case STATE_READYTORECEIVE:
				b3DMode = false;
				g.setColor(colorSelected);
				break;
		}
		for (int cntr = 0; cntr < m_nBorderSize; ++cntr)
			g.draw3DRect(m_nX + cntr, m_nY + cntr, getWidth() - cntr*2,
						getHeight() - cntr*2, b3DMode);
		g.fillRect(m_nX + m_nBorderSize, m_nY + m_nBorderSize,
					getWidth() - m_nBorderSize*2+1, getHeight()- m_nBorderSize*2+1);

		paintDrag(g, observer);
    g.setColor(oldColor);
		m_bItemNeedsToBeDrawn = true;
		return 0;
	}

	// Paint the dragged impage one the screen
	public void paintDrag(Graphics g, ImageObserver observer) {
    if (!isVisible())
      return;
		if (m_pItem != null)
			m_pItem.relativePaint(g, m_nItemX-m_pItem.getWidth()/2, m_nItemY-m_pItem.getHeight()/2, observer);
	}
}