/*
 * Common java library used for Coffee Quest 4 & Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.common.zones;

import java.awt.*;
import java.awt.image.*;

/**
 * This is the root class for all zones components.  This class is designed
 * to handle areas of the screen and uses a mask mechanism to determin if
 * messages should be sent to the listener.
 *   
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: March 2, 1998
 */
public class Zone {
// *** CONSTANTS ***
	public static final int MASK_HIGHLIGHT = 1;
	public static final int MASK_SELECTED = 2;
	public static final int MASK_DRAGGED = 4;
	public static final int MASK_RELEASED = 8;
	public static final int MASK_DROPABLE = 16; // can drop on RECEIVER zones
	public static final int MASK_RECEIVER = 32;
	public static final int MASK_DISABLED = 64;
	public static final int MASK_OWNERREDRAW = 128;	// component that owns zone draws it
	public static final int MASK_TEST = 256;
  public static final int MASK_HIDDEN = 512;
			// MASK states for specifying zone type
	public static final int MASKS_CLICKABLE = 10;
	public static final int MASKS_ALERTBUTTON = 11;
	public static final int MASKS_DRAGDROP = 14;
	public static final int MASKS_ROBUST = 15;
			// Common mask combinations
	public static final int STATE_INALIDATED = -2;
	public static final int STATE_DISABLED = -1;
	public static final int STATE_ACTIVE = 0;
	public static final int STATE_HIGHLIGHTED = 1;
	public static final int STATE_SELECTED = 2;
	public static final int STATE_DRAGGED = 3;
	public static final int STATE_READYTORECEIVE = 4;
			// Potential zone states
// *** VARIABLES ***
	int m_nID;
			// ID of the component
	int m_nX, m_nY, m_nX2, m_nY2;
			// Positional information
	int m_nState, m_nUpdateState;
			// Determins active state of the zone
	int m_nMessageMask;
			// Determins which events get sent to the listener
      // Also used to hold the hidden flag

// *** CONSTRUCTOR ***
	public Zone(int nID, int nX, int nY, int nW, int nH, int nMask) {
		m_nID = nID;
		m_nX = nX;
		m_nY = nY;
		m_nX2 = nX + nW;
		m_nY2 = nY + nH;
		m_nMessageMask = nMask;
		if (checkMessageMask(MASK_DISABLED))
			m_nState = STATE_DISABLED;
		else
			m_nState = STATE_ACTIVE;
	}

// *** MODIFICATION ***
	// Moves the zone to a new location
	public void move(int nX, int nY) {
    int nW = getWidth();
    int nH = getHeight();
		m_nX = nX;
		m_nY = nY;
		m_nX2 = nX + nW;
		m_nY2 = nY + nH;
	}

	// Resizes the zone
	public void resize(int nW, int nH) {
		m_nX2 = m_nX + nW;
		m_nY2 = m_nY + nH;
	}

	// Sets the state of the zone
	public void setState(int nState) {
		m_nState = nState;
	}

	// Changes the mask value for the zone
	public void setMask(int nMask) {
		m_nMessageMask = nMask;
	}

	// Enable the zone
	public void enable() {
		m_nState = STATE_ACTIVE;
	}

	// Disable the zone
	public void disable() {
		m_nState = STATE_DISABLED;
	}

  // Sets the visiblitiy state of the zone.
  // Setting to false also disables, while setting to true enables.
  public void setVisible(boolean b) {
    if (b) {
      m_nMessageMask &= (~MASK_HIDDEN);
  		m_nState = STATE_ACTIVE;
    }
    else {
      m_nMessageMask |= MASK_HIDDEN;
  		m_nState = STATE_DISABLED;
    }
  }

// *** INFORMATION ***
  // Returns true if zone is visible.
  public final boolean isVisible() {
    if ( (m_nMessageMask & MASK_HIDDEN) == MASK_HIDDEN)
      return false;
    else
      return true;
  }

	// returns the ID of this zone
	public final int getID() {
		return m_nID;
	}

	// returns the X coordinate of this zone
	public final int getX() {
		return m_nX;
	}

	// returns the Y coordinate of this zone
	public final int getY() {
		return m_nY;
	}

	// returns the X2 coordinate of this zone
	public final int getX2() {
		return m_nX2;
	}

	// returns the Y2 coordinate of this zone
	public final int getY2() {
		return m_nY2;
	}

	// returns the width of this zone
	public final int getWidth() {
		return m_nX2 - m_nX;
	}

	// returns the height of this zone
	public final int getHeight() {
		return m_nY2 - m_nY;
	}

	// Checks the mask for specific flags
	public boolean checkMessageMask(int nMask) {
		return ((m_nMessageMask & nMask) == nMask);
	}

	// checks to see if this zone can be dragged & dropped
	public boolean canBeDropped() {
		return ((m_nState != STATE_DISABLED) && checkMessageMask(MASK_DROPABLE));
	}

	// Sees if this zone HAS JUST BEEN dropped
	public boolean hasBeenDropped() {
		return ( canBeDropped() && (m_nState == STATE_DRAGGED));
	}

	// sees if zone is ready to receive (and can receive) drop messages
	public boolean canRecieve() {
		return (m_nState == STATE_READYTORECEIVE);
	}

	public int getState() {
		return m_nState;
	}

	// Returns true if there is a dirty rectangle due to dragging
	public boolean checkDragDrawState() {
		/* Placeholder.  Used by Drag/drop funcions */
		return false;
	}

	// Used to tell class that the dirty area of the drag has been cleaned up
	public void clearDragDrawState() {
		/* Placeholder.  Used by Drag/drop funcions */
	}

	// returns rectangle with coordinates of the dirty rectangle due to dragging
	public Rectangle getDragDropDirtyRect() {
		/* Placeholder.  Used by Drag/drop funcions */
		return new Rectangle();
	}

// *** INTERSECTION CHECKING FUNCTIONS ***
	// Check to see if point intersects zone
	// returns true if it does
	public boolean checkPoint(int nX, int nY) {
		return ( (nX > m_nX) && (nY > m_nY) && (nX < m_nX2) && (nY < m_nY2) );
	}

	// Check to see if rectangle intersects zone
	// returns true if it does
	public boolean checkRectangle(int x1, int y1, int x2, int y2) {
		return ( (x2 > m_nX) && (x1 < m_nX2) && (y2 > m_nY) && (y1 < m_nY2) );
	}

	// Check to see if Zone intersects zone
	// returns true if it does
	public boolean checkZoneBounding(Zone z) {
		return checkRectangle(z.getX(), z.getY(), z.getX2(), z.getY2());
	}

// *** MESSAGE HANDLING ***
	// Handle potential mouse click
	// returns Event ID, 0 for no event
	public int mouseDown(int nX, int nY) {
		if (m_nState != STATE_DISABLED) {
			if (checkPoint(nX, nY)) {
				m_nState = STATE_SELECTED;
				if (checkMessageMask(MASK_SELECTED) )
					return ZoneListener.ZN_SELECTED;
			}
		}

		return 0;
	}

	// Handle potential mouse release
	// returns Event ID, 0 for no event
	public int mouseUp(int nX, int nY) {
		if (m_nState != STATE_DISABLED) {
			if (checkPoint(nX, nY)) {
				if (m_nState != STATE_DRAGGED)
					if (checkMessageMask(MASK_RECEIVER) )
						m_nState = STATE_READYTORECEIVE;
					else
						m_nState = STATE_ACTIVE;
				if (checkMessageMask(MASK_RELEASED) )
					return ZoneListener.ZN_RELEASED;
			}
			else
				if ( (m_nState == STATE_SELECTED) || (m_nState == STATE_DRAGGED)) {
					if (m_nState == STATE_SELECTED)
						m_nState = STATE_ACTIVE;
					if (checkMessageMask(MASK_RELEASED) )
						return ZoneListener.ZN_OUTOFZONERELEASE;
				}
		}

		return 0;
	}

	// Handle potential highlight/unhighlight
	// returns Event ID, 0 for no event
	public int mouseMove(int nX, int nY) {
		if (m_nState != STATE_DISABLED) {
			if (checkPoint(nX, nY)) {
				if (m_nState != STATE_HIGHLIGHTED) {
					m_nState = STATE_HIGHLIGHTED;
					if (checkMessageMask(MASK_HIGHLIGHT) )
						return ZoneListener.ZN_HIGHLIGHTED;
				}
			}
			else {
				if (m_nState == STATE_HIGHLIGHTED) {
					m_nState = STATE_ACTIVE;
					if (checkMessageMask(MASK_HIGHLIGHT) )
						return ZoneListener.ZN_UNHIGHLIGHTED;
				}
			}
		}

		return 0;
	}

	// Handles potential zone drag
	// returns Event ID, 0 for no event
	public int mouseDrag(int nX, int nY) {
		if (m_nState != STATE_DISABLED) {
			if ( (m_nState == STATE_SELECTED) || (m_nState == STATE_DRAGGED) ) {
				if (checkMessageMask(MASK_DRAGGED) ) {
					m_nState = STATE_DRAGGED;
					return ZoneListener.ZN_DRAGGED;
				}
			}
			else
				return mouseMove(nX, nY);
		}

		return 0;
	}

	// Clears the dragged state if zone has been dragged or is receiving
	public void clearDrag() {
		if ( (m_nState == STATE_DRAGGED) || (m_nState == STATE_READYTORECEIVE) )
			m_nState = STATE_ACTIVE;
	}

// *** DRAWING ***
	// Fully draws the component
	public int paint(Graphics g, ImageObserver observer) {
    if (!isVisible())
      return 0;
		if (checkMessageMask(MASK_TEST) ) {
			switch (m_nState) {
				case STATE_DISABLED:
					g.setColor(Color.gray);
					break;
				case STATE_ACTIVE:
					g.setColor(Color.blue);
					break;
				case STATE_HIGHLIGHTED:
					g.setColor(Color.red);
					break;
				case STATE_SELECTED:
					g.setColor(Color.magenta);
					break;
				case STATE_DRAGGED:
				case STATE_READYTORECEIVE:
					g.setColor(Color.green);
					break;
			}
			g.drawRect(m_nX, m_nY, getWidth(), getHeight());
		}
		m_nUpdateState = m_nState;
		if (checkMessageMask(MASK_OWNERREDRAW))
			return ZoneListener.ZN_REPAINTZONE;
		else
			return 0;
	}

	// Draws the component if the indicated area overlaps the zone
	public int paint(Graphics g, int x1, int y1, int x2, int y2, ImageObserver observer) {
		if (checkRectangle(x1,y1,x2,y2))
			return paint(g, observer);
		else
			return 0;
	}

	// If rectangle part of zone, Invalidate the zone
	public void invalidateRect(int x1, int y1, int x2, int y2) {
		if (checkRectangle(x1,y1,x2,y2))
			m_nUpdateState = STATE_INALIDATED;
	}

	// Invalidate's (tells update zone needs redrawing) zone.
	public void invalidate() {
		m_nUpdateState = STATE_INALIDATED;
	}

	// Draws the component if the state has changed since last update
	public int update(Graphics g, ImageObserver observer) {
		if (m_nUpdateState != m_nState)
			return paint(g, observer);
		else
			return 0;
	}

	public void paintDrag(Graphics g, ImageObserver observer) {
		/* Here for classes that support Drag and Drop */
	}
}