/*
 * Common java library used for Coffee Quest 4 & Wilmags Lab
 * Copyright (c) 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.common.zones;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;

/**
 * This is the class that handles the zones.  The user adds the zones to
 * this class, then sets this class as the component's mouse and mousemotion
 * listener.  Zones that generate cause the adapter to call the zonelistener
 * specified when it is created (ussually the class that created the adapter).
 *   
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: March 2, 1998
 */
public class ZoneAdapter implements MouseListener, MouseMotionListener {
// *** CONSTANTS

// *** VARIABLES
	Vector listZones;
	ZoneListener zoneListener;

  int nMouseX, nMouseY;

// *** CONSTRUCTOR ***
	public ZoneAdapter(ZoneListener zl) {
		zoneListener = zl;
		listZones = new Vector(10,10);
	}

// *** SETUP & MODIFICATION METHODS ***
	// Add a zone to the list of zones to monitor
	public void add(Zone z) {
		listZones.addElement(z);
	}

	// remove all the zones from the list of zones to monitor
	public void clearZones() {
		listZones.removeAllElements();
	}

	// Removes zone
	public void removeZone(Zone z) {
		listZones.removeElement(z);
	}

	// Goes through all the zones painting them
	// DOES NOT handle drag/drop repainting!
	public void paintZones(Graphics g, ImageObserver observer) {
		Zone z;
		int nEvent;

		for (Enumeration e = listZones.elements() ; e.hasMoreElements() ;) {
			z = (Zone)(e.nextElement());
			nEvent = z.paint(g, observer);
			if (nEvent > 0)
				zoneListener.zoneMessage(z.getID(), nEvent);
		}
	}

	// Goes through all the zones painting any that intersect the bounds
	public void paintZones(Graphics g, int x1, int y1, int x2, int y2, ImageObserver observer) {
		Zone z;
		int nEvent;

		for (Enumeration e = listZones.elements() ; e.hasMoreElements() ;) {
			z = (Zone)(e.nextElement());
			nEvent = z.paint(g, x1, y1, x2, y2, observer);
			if (nEvent > 0)
				zoneListener.zoneMessage(z.getID(), nEvent);
		}
	}

	// Goes through all the zones Invalidating a rectangular regeion
	public void invalidateRect(int x1, int y1, int x2, int y2) {
		Zone z;

		for (Enumeration e = listZones.elements() ; e.hasMoreElements() ;) {
			z = (Zone)(e.nextElement());
			z.invalidateRect(x1, y1, x2, y2);
		}
	}

	// Goes through all the zones updating them
	// DOES NOT handle drag/drop repainting!
	public void updateZones(Graphics g, ImageObserver observer) {
		Zone z;
		int nEvent;

		for (Enumeration e = listZones.elements() ; e.hasMoreElements() ;) {
			z = (Zone)(e.nextElement());
			nEvent = z.update(g, observer);
			if (nEvent > 0)
				zoneListener.zoneMessage(z.getID(), nEvent);
		}
	}

	// Goes through all the zones updating them.  Handles dragging by filling
	// Non-Zone areas with provided color while refreshing affected zones.
	public void updateZones(Graphics g, Color colorBackground, ImageObserver observer) {
		Zone z;
		int nEvent;
		Rectangle rect;

		// First Pass - Fill dirty rectangles with background color
		for (Enumeration e = listZones.elements() ; e.hasMoreElements() ;) {
			z = (Zone)(e.nextElement());
			if (z.checkDragDrawState()){
				rect = z.getDragDropDirtyRect();
				invalidateRect(rect.x, rect.y, rect.x+rect.width, rect.y+rect.height);
				g.setColor(colorBackground);
				g.fillRect(rect.x, rect.y, rect.width, rect.height);
			}
		}

		// Second Pass - Paint the zones
		for (Enumeration e = listZones.elements() ; e.hasMoreElements() ;) {
			z = (Zone)(e.nextElement());
			nEvent = z.update(g, observer);
			if (nEvent > 0)
				zoneListener.zoneMessage(z.getID(), nEvent);
		}

		// Third Pass - Paint the dragged items
		for (Enumeration e = listZones.elements() ; e.hasMoreElements() ;) {
			z = (Zone)(e.nextElement());
			if (z.checkDragDrawState()){
				z.paintDrag(g, observer);
				z.clearDragDrawState();
			}
		}
	}

// *** MOUSE LISTENER METHODS ***
	// Invoked when the mouse has been clicked on a component.
	public void mouseClicked(MouseEvent e) {
		/* IGNORE THIS EVENT */
	}

	// Invoked when a mouse button has been pressed on a component.
	public void mousePressed(MouseEvent e) {
		int nEvent;
		Zone z;

		for (Enumeration en = listZones.elements() ; en.hasMoreElements() ;) {
			z = (Zone)(en.nextElement());
			nEvent = z.mouseDown(e.getX(), e.getY());
			if (nEvent > 0)
				zoneListener.zoneMessage(z.getID(), nEvent);

		}
	}

	//  Invoked when a mouse button has been released on a component.
	public void mouseReleased(MouseEvent e) {
		int nEvent;
		Zone z, z2;

		for (Enumeration en = listZones.elements() ; en.hasMoreElements() ;) {
			z = (Zone)(en.nextElement());
			nEvent = z.mouseUp(e.getX(), e.getY());
			if (nEvent > 0)
				zoneListener.zoneMessage(z.getID(), nEvent);
			if (z.canRecieve()) {
				for (Enumeration e2 = listZones.elements() ; e2.hasMoreElements() ;) {
					z2 = (Zone)(e2.nextElement());
					if (z2.hasBeenDropped()) {
						zoneListener.zoneDragDrop(z.getID(), z2.getID());
					}
				}
			}
		}
		for (Enumeration en = listZones.elements() ; en.hasMoreElements() ;) {
			z = (Zone)(en.nextElement());
			z.clearDrag();
		}

	}

	// Invoked when the mouse enters a component.
	public void mouseEntered(MouseEvent e) {
		/* IGNORE THIS EVENT */
	}

	// Invoked when the mouse exits a component.
	public void mouseExited(MouseEvent e) {
		/* IGNORE THIS EVENT */
	}

// *** MOUSE MOTION LISTENER EVENTS ***
	//Invoked when a mouse button is pressed on a component and then dragged. Mouse drag events will continue to be delivered to the component where the first originated until the mouse button is released (regardless of whether the mouse position is within the bounds of the component).
	public void mouseDragged(MouseEvent e) {
		int nEvent;
		Zone z;

    nMouseX = e.getX();
    nMouseY = e.getY();
		for (Enumeration en = listZones.elements() ; en.hasMoreElements() ;) {
			z = (Zone)(en.nextElement());
			nEvent = z.mouseDrag(nMouseX, nMouseY);
			if (nEvent > 0)
				zoneListener.zoneMessage(z.getID(), nEvent);

		}
	}

	//Invoked when the mouse button has been moved on a component (with no buttons no down).
	public void mouseMoved(MouseEvent e) {
		int nEvent;
		Zone z;

    nMouseX = e.getX();
    nMouseY = e.getY();
		for (Enumeration en = listZones.elements() ; en.hasMoreElements() ;) {
			z = (Zone)(en.nextElement());
			nEvent = z.mouseMove(nMouseX, nMouseY);
			if (nEvent > 0)
				zoneListener.zoneMessage(z.getID(), nEvent);

		}
	}

  // Fake drag - forced by user to make sure dragging not disrupted durring
  // screen altering event (such as animation)
	public void fakeDrag() {
		int nEvent;
		Zone z;

		for (Enumeration en = listZones.elements() ; en.hasMoreElements() ;) {
			z = (Zone)(en.nextElement());
			nEvent = z.mouseDrag(nMouseX, nMouseY);
			if (nEvent > 0)
				zoneListener.zoneMessage(z.getID(), nEvent);
		}
	}

}

